import 'package:flutter/material.dart';
import 'package:vivanta_connect_flutter/data/brand.dart';
import 'package:vivanta_connect_flutter/services/http.dart';
import 'package:vivanta_connect_flutter/services/preferences.dart';
import 'package:vivanta_connect_flutter/styles/colors.dart';
import 'package:vivanta_connect_flutter/styles/fonts.dart';
import 'package:vivanta_connect_flutter/views/loading.dart';
import 'package:vivanta_connect_flutter/views/webview.dart';
import 'package:vivanta_connect_flutter/widgets/brand_item.dart';

class Brands extends StatefulWidget {
  const Brands({Key? key}) : super(key: key);

  @override
  State<Brands> createState() => _BrandsState();
}

class _BrandsState extends State<Brands> {
  bool _loading = false;
  bool showSearch = false;
  String _apiKey = '';
  String _customerId = '';
  String _externalUserId = '';
  List<Brand> _webBrands = [];
  TextEditingController searchController = TextEditingController();
  FocusNode searchFocus = FocusNode();
  final http = Http();
  final storage = Preferences();

  @override
  void initState() {
    _getBrands();
    searchController.addListener(() => setState(() {}));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return _loading
        ? const Loading()
        : Scaffold(
            backgroundColor: VivantaColors.background,
            appBar: AppBar(
              backgroundColor: VivantaColors.background,
              elevation: 0,
              title: showSearch
                  ? TextFormField(
                      decoration: InputDecoration(
                        hintText: "Find brand",
                        hintStyle: VivantaFonts.title.copyWith(height: 1),
                        labelStyle: VivantaFonts.title.copyWith(height: 1),
                        contentPadding: const EdgeInsets.all(4),
                        isDense: true,
                      ),
                      style: VivantaFonts.title,
                      controller: searchController,
                      focusNode: searchFocus,
                    )
                  : const Text(
                      "Connect your device",
                      style: VivantaFonts.title,
                    ),
              actions: [
                Padding(
                  padding: const EdgeInsets.only(right: 16),
                  child: GestureDetector(
                    onTap: () {
                      setState(() {
                        showSearch = !showSearch;
                        if (showSearch) {
                          searchFocus.requestFocus();
                        } else {
                          searchController.text = '';
                          searchFocus.unfocus();
                        }
                      });
                    },
                    child: Icon(
                      showSearch ? Icons.close : Icons.search,
                      color: VivantaColors.text,
                      size: 24,
                    ),
                  ),
                ),
              ],
              leading: GestureDetector(
                onTap: () {
                  Navigator.of(context).maybePop();
                },
                child: const Icon(
                  Icons.arrow_back_ios,
                  color: VivantaColors.text,
                  size: 24,
                ),
              ),
            ),
            body: GridView.count(
              crossAxisCount: 2,
              children: [
                ..._getBrandItems(),
              ],
            ),
          );
  }

  Future<void> _getBrands() async {
    setState(() {
      _loading = true;
    });
    _apiKey = await storage.getApiKey();
    _customerId = await storage.getCustomerId();
    _externalUserId = await storage.getExternalUserId();
    final token = await storage.getToken();
    _webBrands = await http.getBrands(token);
    setState(() {
      _loading = false;
    });
  }

  List<BrandItem> _getBrandItems() {
    final filteredBrands = _webBrands.where((element) => element.name!
        .toLowerCase()
        .contains(searchController.text.toLowerCase()));

    return filteredBrands
        .map<BrandItem>(
          (brand) => BrandItem(
            brand: brand,
            onTap: (brand) {
              final url =
                  'https://connect.vivanta.io/native/$_apiKey/$_customerId/$_externalUserId/${brand.id}';
              Navigator.of(context).push(
                MaterialPageRoute(
                  builder: (context) => Webview(
                    brand: brand,
                    url: url,
                  ),
                ),
              );
            },
          ),
        )
        .toList();
  }

  @override
  void dispose() {
    searchController.dispose();
    searchFocus.dispose();
    super.dispose();
  }
}
